@@ -30,7 +30,7 @@ class NMEAHandler:
         """
 
         self.__app = app  # Reference to main application class
-        self.__master = self.__app.get_master()  # Reference to root class (Tk)
+        self.__master = self.__app.appmaster  # Reference to root class (Tk)
 
         self._raw_data = None
         self._parsed_data = None
