@@ -61,6 +61,10 @@ STOPDATA = "Serial reader process stopped"
 NOTCONN = "Not connected"
 UBXPOLL = "Polling current UBX configuration..."
 MONSPANERROR = "Enable or poll MON-SPAN message"
+VERCHECK = "Newer version of PyGPSClient available: {}"
+NULLSEND = "Nothing to send"
+CONFIGOK = "{} command accepted"
+CONFIGBAD = "{} command rejected"
 
 
 # Menu text
@@ -76,6 +80,7 @@ MENUHIDEUBX = "Hide UBX Config"
 MENUSHOWUBX = "Show UBX Config"
 MENUUBXCONFIG = "UBX Configuration Dialog"
 MENUNTRIPCONFIG = "NTRIP Configuration Dialog"
+MENUSPARTNCONFIG = "SPARTN Configuration Dialog"
 MENUHOWTO = "How To"
 MENUABOUT = "About"
 MENUHELP = "Help"
@@ -91,8 +96,9 @@ BTNRST = "Reset"
 # Label text
 LBLPROTDISP = "Protocols Shown"
 LBLDATADISP = "Console Display"
-LBLUBXCONFIG = "UBX\nConfig"
-LBLNTRIPCONFIG = "NTRIP\nClient"
+LBLUBXCONFIG = "UBX Config"
+LBLNTRIPCONFIG = "NTRIP Client"
+LBLSPARTNCONFIG = "SPARTN Client"
 LBLNTRIPGGAINT = "GGA Interval s"
 LBLCTL = "Controls"
 LBLSET = "Settings"
@@ -121,10 +127,22 @@ LBLGGALIVE = "Receiver"
 LBLGGAFIXED = "Fixed Reference"
 LBLUDPORT = "USER-DEFINED PORT"
 LBLCFGRECORD = "CFG Configuration Load/Save/Record"
+LBLSPTNCURR = "CURRENT SPARTN KEY:"
+LBLSPTNNEXT = "NEXT SPARTN KEY:"
+LBLSPTNKEY = "Key as hexadecimal"
+LBLSPTNDAT = "Valid from YYYYMMDD"
+LBLSPTNUPLOAD = "Upload keys?"
+LBLSPTNFP = "Configure receiver?"
+LBLSPTNNMEA = "Disable NMEA?"
+LBLSPARTNIP = "IP CORRECTION CONFIGURATION (MQTT)"
+LBLSPARTNLB = "L-BAND CORRECTION CONFIGURATION (D9*)"
+LBLSPARTNGN = "GNSS RECEIVER CONFIGURATION (F9*)"
+
 
 # Dialog text
 DLGUBXCONFIG = "UBX Configuration"
 DLGNTRIPCONFIG = "NTRIP Client Configuration"
+DLGSPARTNCONFIG = "SPARTN Client Configuration"
 DLGGPXVIEWER = "GPX Track Viewer"
 DLGABOUT = "PyGPSClient"
 DLGHOWTO = "How To Use PyGPSCLient"
